/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.pdfdisplaysettings;


import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.RadioGroup;
import android.widget.Switch;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatRadioButton;
import androidx.fragment.app.FragmentManager;

import com.compdfkit.tools.R;
import com.compdfkit.tools.pdfview.CPDFView;
import com.compdfkit.tools.utils.dialog.DialogFragmentUtil;
import com.compdfkit.tools.utils.view.CToolBar;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.android.material.switchmaterial.SwitchMaterial;

public class CPDFDisplaySettingDialogFragment extends BottomSheetDialogFragment implements RadioGroup.OnCheckedChangeListener, CompoundButton.OnCheckedChangeListener {


    /**
     * Bright mode, white color
     */
    public static final int READER_LIGHT_MODE = 0xFFFFFFFF;
    /**
     * Dark mode, dark gray color
     */
    public static final int READER_DARK_MODE = 0xFF333333;
    /**
     * Sepia mode, a color similar to the brown color of paper
     */
    public static final int READER_SEPIA_MODE = 0xFFFFEFBE;
    /**
     * Eye protection mode, light green color
     */
    public static final int READER_RESEDA_MODE = 0XFFCDE6D0;


    private CToolBar toolBar;
    private RadioGroup readerModeRadioGroup;
    private AppCompatRadioButton rbSinglePage;
    private AppCompatRadioButton rbDoublePage;
    private AppCompatRadioButton rbCoverDoublePage;

    private Switch swIsContinue;
    private Switch swIsVertical;
    private Switch swIsCropMode;
    private RadioGroup readerBgRadioGroup;
    private AppCompatRadioButton rbLightMode;
    private AppCompatRadioButton rbDarkMode;
    private AppCompatRadioButton rbSepiaMode;

    private AppCompatRadioButton rbResedaMode;


    private CPDFView cpdfView;


    public static void show(FragmentManager fragmentManager, CPDFView cpdfView) {
        CPDFDisplaySettingDialogFragment displaySettingDialogFragment = new CPDFDisplaySettingDialogFragment();
        displaySettingDialogFragment.setCpdfReaderView(cpdfView);
        displaySettingDialogFragment.show(fragmentManager, "displaySettingsDialog");
    }

    public void setCpdfReaderView(CPDFView cpdfView) {
        this.cpdfView = cpdfView;
    }

    @Override
    public void onStart() {
        super.onStart();
        BottomSheetBehavior<View> behavior = BottomSheetBehavior.from((View) getView().getParent());
        DialogFragmentUtil.setBottomSheetDialogFragmentFullScreen(getDialog(), behavior);
    }


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.tools_pdf_display_settings_dialog_fragment, container, false);
        toolBar = rootView.findViewById(R.id.tool_bar);
        toolBar.setBackBtnClickListener(v -> dismiss());
        readerModeRadioGroup = rootView.findViewById(R.id.radio_group_reader_mode);
        rbSinglePage = rootView.findViewById(R.id.r_btn_single_page);
        rbDoublePage = rootView.findViewById(R.id.r_btn_double_page);
        rbCoverDoublePage = rootView.findViewById(R.id.r_btn_cover_double_page);
        swIsContinue = rootView.findViewById(R.id.sw_is_continue);
        swIsVertical = rootView.findViewById(R.id.sw_is_vertical);
        swIsCropMode = rootView.findViewById(R.id.sw_is_crop);
        readerBgRadioGroup = rootView.findViewById(R.id.radio_group_reader_bg);
        rbLightMode = rootView.findViewById(R.id.r_btn_light_mode);
        rbDarkMode = rootView.findViewById(R.id.r_btn_dark_mode);
        rbSepiaMode = rootView.findViewById(R.id.r_btn_sepia_mode);
        rbResedaMode = rootView.findViewById(R.id.r_btn_reseda_mode);

        swIsContinue.setOnCheckedChangeListener(this);
        swIsVertical.setOnCheckedChangeListener(this);
        swIsCropMode.setOnCheckedChangeListener(this);

        readerModeRadioGroup.setOnCheckedChangeListener(this);
        readerBgRadioGroup.setOnCheckedChangeListener(this);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (cpdfView != null) {
            com.compdfkit.ui.reader.CPDFReaderView readerView = cpdfView.getCPdfReaderView();

            if (readerView.isDoublePageMode()) {
                if (readerView.isCoverPageMode()) {
                    rbCoverDoublePage.setChecked(true);
                } else {
                    rbDoublePage.setChecked(true);
                }
            } else {
                rbSinglePage.setChecked(true);
            }

            swIsContinue.setChecked(readerView.isContinueMode());
            swIsVertical.setChecked(readerView.isVerticalMode());
            swIsCropMode.setChecked(readerView.isCropMode());


            switch (readerView.getReadBackgroundColor()) {
                case READER_LIGHT_MODE:
                    rbLightMode.setChecked(true);
                    break;
                case READER_DARK_MODE:
                    rbDarkMode.setChecked(true);
                    break;
                case READER_SEPIA_MODE:
                    rbSepiaMode.setChecked(true);
                    break;
                case READER_RESEDA_MODE:
                    rbResedaMode.setChecked(true);
                    break;
                default:
                    break;
            }
        }
    }

    @Override
    public void onCheckedChanged(RadioGroup group, int checkedId) {
        if (checkedId == R.id.r_btn_light_mode) {
            setReaderBackgroundColor(READER_LIGHT_MODE);
        } else if (checkedId == R.id.r_btn_dark_mode) {
            setReaderBackgroundColor(READER_DARK_MODE);
        } else if (checkedId == R.id.r_btn_sepia_mode) {
            setReaderBackgroundColor(READER_SEPIA_MODE);
        } else if (checkedId == R.id.r_btn_reseda_mode) {
            setReaderBackgroundColor(READER_RESEDA_MODE);
        } else if (checkedId == R.id.r_btn_single_page) {

            if (cpdfView != null) {
                cpdfView.getCPdfReaderView().setDoublePageMode(false);
                cpdfView.getCPdfReaderView().setCoverPageMode(false);
            }

        } else if (checkedId == R.id.r_btn_double_page) {
            if (cpdfView != null) {
                cpdfView.getCPdfReaderView().setDoublePageMode(true);
                cpdfView.getCPdfReaderView().setCoverPageMode(false);
            }
        } else if (checkedId == R.id.r_btn_cover_double_page) {
            if (cpdfView != null) {
                cpdfView.getCPdfReaderView().setDoublePageMode(true);
                cpdfView.getCPdfReaderView().setCoverPageMode(true);
            }
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if (buttonView.getId() == R.id.sw_is_vertical) {
            if (cpdfView != null) {
                cpdfView.getCPdfReaderView().setVerticalMode(isChecked);
            }
        } else if (buttonView.getId() == R.id.sw_is_continue) {
            if (cpdfView != null) {
                cpdfView.getCPdfReaderView().setContinueMode(isChecked);
            }
        } else if (buttonView.getId() == R.id.sw_is_crop) {
            if (cpdfView != null) {
                cpdfView.getCPdfReaderView().setCropMode(isChecked);
            }
        }
    }

    private void setReaderBackgroundColor(int color) {
        if (cpdfView != null) {
            cpdfView.getCPdfReaderView().setReadBackgroundColor(color);
        }
    }
}
