/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.pdfview;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.widget.RelativeLayout;

import androidx.annotation.DrawableRes;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.tools.R;
import com.compdfkit.tools.pdfview.dialog.CGotoPageDialog;
import com.compdfkit.tools.pdfview.view.CPDFPageIndicatorView;
import com.compdfkit.tools.pdfview.view.CPDFSliderBarView;
import com.compdfkit.tools.utils.viewutils.DimensUtils;
import com.compdfkit.ui.reader.CPDFReaderView;
import com.compdfkit.ui.reader.IReaderViewCallback;
import com.compdfkit.ui.widget.CPDFSlideBar;


/**
 * compdfkit sdk CPDFReaderView ctrl view <br/>
 *
 *
 * ︳--------------------------------︳<BR/>
 * ︳　　　　　　　    　　 　 |---------︳<BR/>
 * ︳　　　　　　　　　　  　|sliderbar︳<BR/>
 * ︳　　　　　　　　　    　　|---------︳<BR/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　 　cpdfReaderView　　      　　︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳　　　　　　　　　　 　　     　　　 ︳<br/>
 * ︳|---------|　　　　　　 　    　　　 ︳<br/>
 * ︳|indicator|　　　　　　 　    　　　 ︳<br/>
 * ︳|---------|　　　　　　 　    　　　 ︳<br/>
 * ︳--------------------------------︳<br/>
 * <p>
 * STEP 1:<br/>
 * Use CPDFReaderView in xml layout <br/>
 * <br/>
 * com.compdfkit.tools.pdfview.CPDFView<br/>
 * android:id="@+id/pdf_reader_view" <br/>
 * android:layout_width="match_parent"<br/>
 * android:layout_height="match_parent"<br/>
 * />
 *
 * <p/>
 * STEP 2:
 * open pdf file <br/>
 * use {@link CPDFView#openPdfFile(String pdfFilePath)}
 *
 * <p/>
 * Congratulations you have completed the pdf file display!!!
 * <br/>
 *
 * <br/>
 * custom attributes:<br/>
 * app:tools_enable_slider_bar="true|false"  <br/>
 * <p/>
 * app:tools_slider_bar_position="left|right"
 * <p/>
 * app:tools_slider_bar_icon="@drawable/xxx"
 * <p/>
 * app:tools_slider_bar_thumbnail_width="120dp"
 * <p/>
 * app:tools_slider_bar_thumbnail_hei="200dp"
 * <p/>
 * app:tools_enable_page_indicator="true|false"
 */
public class CPDFView extends RelativeLayout implements IReaderViewCallback {

    private CPDFReaderView cPdfReaderView;

    private int currentPageIndex = 1;

    private CPDFSlideBar slideBar;

    private boolean enableSliderBar = true;

    private CPDFSlideBar.SlideBarPosition slideBarPosition = CPDFSlideBar.SlideBarPosition.RIGHT;

    private int sliderBarThumbnailWidth = 314;
    private int sliderBarThumbnailHeight = 444;

    @DrawableRes
    private int sliderBarIconResId = R.drawable.tools_ic_pdf_slider_bar;
    private CPDFPageIndicatorView indicatorView;

    private boolean enablePageIndicator = true;

    public CPDFView(@NonNull Context context) {
        this(context, null);
    }

    public CPDFView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPDFView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initAttr(context, attrs);
        initCPDFReaderView();
    }


    private void initAttr(Context context, AttributeSet attributeSet) {
        try {
            TypedArray typedArray = context.obtainStyledAttributes(attributeSet, R.styleable.CPDFReaderView);
            if (typedArray != null) {
                enableSliderBar = typedArray.getBoolean(R.styleable.CPDFReaderView_tools_enable_slider_bar, true);
                int sliderBarPositionEnum = typedArray.getInt(R.styleable.CPDFReaderView_tools_slider_bar_position, 1);
                if (sliderBarPositionEnum == 0) {
                    slideBarPosition = CPDFSlideBar.SlideBarPosition.LEFT;
                } else {
                    slideBarPosition = CPDFSlideBar.SlideBarPosition.RIGHT;
                }
                sliderBarThumbnailWidth = typedArray.getDimensionPixelOffset(R.styleable.CPDFReaderView_tools_slider_bar_thumbnail_width, 314);
                sliderBarThumbnailHeight = typedArray.getDimensionPixelOffset(R.styleable.CPDFReaderView_tools_slider_bar_thumbnail_height, 444);
                sliderBarIconResId = typedArray.getResourceId(R.styleable.CPDFReaderView_tools_slider_bar_icon, R.drawable.tools_ic_pdf_slider_bar);
                enablePageIndicator = typedArray.getBoolean(R.styleable.CPDFReaderView_tools_enable_page_indicator, true);
                typedArray.recycle();
            }
        } catch (Exception e) {

        }

    }


    private void initCPDFReaderView() {
        cPdfReaderView = new com.compdfkit.ui.reader.CPDFReaderView(getContext());
        cPdfReaderView.setDoublePageMode(false);
        cPdfReaderView.setReaderViewCallback(this);
        addView(cPdfReaderView);
    }


    /**
     * This method opens a PDF file located at the given file path.
     *
     * @param pdfFilePath The file path of the PDF file to be opened.
     */
    public void openPdfFile(String pdfFilePath) {
        // Create a new instance of the CPDFDocument class, passing in the current context.
        CPDFDocument cpdfDocument = new CPDFDocument(getContext());
        // Attempt to open the PDF file at the given file path using the open method of the CPDFDocument class.
        CPDFDocument.PDFDocumentError pdfDocumentError = cpdfDocument.open(pdfFilePath);
        // Switch on the result of the open method to handle the different possible outcomes.
        switch (pdfDocumentError) {
            // If the PDF file was successfully opened, set the PDF document of the CPdfReaderView instance to the opened document.
            case PDFDocumentErrorSuccess:
                cPdfReaderView.setPDFDocument(cpdfDocument);
                initCPDFSliderBar();
                addPageIndicator();

                break;
            // If the PDF file requires a password to be opened, do nothing for now.
            case PDFDocumentErrorPassword:

                break;
            // For all other errors, do nothing for now.
            default:
                break;
        }
    }

    public com.compdfkit.ui.reader.CPDFReaderView getCPdfReaderView() {
        return cPdfReaderView;
    }


    @Override
    public void onTapMainDocArea() {

    }

    @Override
    public void onMoveToChild(int pageIndex) {
        currentPageIndex = pageIndex;
        if (slideBar != null) {
            slideBar.setPageIndexByAnimation(pageIndex, 500);
        }
        if (indicatorView != null) {
            indicatorView.setCurrentPageIndex(pageIndex);
        }
    }

    @Override
    public void onEndScroll() {

    }

    @Override
    public void onScrolling() {

    }

    @Override
    public void onRecordLastJumpPageNum(int i) {

    }


    private void initCPDFSliderBar() {
        if (!enableSliderBar) {
            return;
        }
        CPDFSliderBarView cpdfSliderBarView = new CPDFSliderBarView(getContext());
        cpdfSliderBarView.bindCPdfReaderView(this);
        cpdfSliderBarView.initSliderBar(slideBarPosition, sliderBarThumbnailWidth, sliderBarThumbnailHeight);
        cpdfSliderBarView.setSlideBarBitmap(sliderBarIconResId);
        setSliderBar(cpdfSliderBarView);
    }

    public void setSliderBar(CPDFSlideBar cpdfSlideBar) {
        enableSliderBar = true;
        LayoutParams layoutParams = new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.MATCH_PARENT);
        slideBar = cpdfSlideBar;
        slideBar.setLayoutParams(layoutParams);
        addView(slideBar);
    }


    /**
     * Add a page indicator in the lower left corner of the page.
     * After clicking, a page number jump input box will pop up, and you can jump to the corresponding PDF page
     * page input limit: 1 ~ pdf total page
     * @see CPDFPageIndicatorView
     */
    public void addPageIndicator() {
        if (!enablePageIndicator) {
            return;
        }
        indicatorView = new CPDFPageIndicatorView(getContext());
        RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
        int margin = DimensUtils.dp2px(getContext(), 16);
        layoutParams.setMargins(margin, 0, 0, margin);
        indicatorView.setLayoutParams(layoutParams);
        addView(indicatorView);

        int totalPageCount = cPdfReaderView.getPDFDocument().getPageCount();
        indicatorView.setTotalPage(totalPageCount);
        indicatorView.setPageIndicatorClickListener(pageIndex -> {
            CGotoPageDialog dialog = new CGotoPageDialog(getContext());
            dialog.setHintInputText(getContext().getString(R.string.tools_search_header_page) +
                    String.format(" (%d/%d)", 1, totalPageCount));
            dialog.setOnSetPDFDisplayPageIndexListener(page -> {
                if (page <= totalPageCount && page > 0){
                    cPdfReaderView.setDisplayPageIndex(page - 1, true);
                }
            });
            dialog.show();
        });
    }


}
