/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.pdfview.view;


import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;

import com.compdfkit.tools.R;
import com.compdfkit.tools.utils.viewutils.ViewUtils;


/**
 * pdf ui tool bar <br/>
 * ︳------------------------------------------------︳<br/>
 * ︳　　　　　　　　title　　　　　icon1  icon2 icon3   ︳<br/>
 * ︳------------------------------------------------︳<br/>
 * icon1: searchIcon <br/>
 * icon2: boTaIcon <br/>
 * icon3: moreIcon <br/>
 * <p/>
 * use samples:<br/>
 * com.compdfkit.tools.pdfview.view.CPDFToolBar <br/>
 *     android:id="@+id/pdf_tool_bar" <br/>
 *     android:layout_width="match_parent" <br/>
 *     android:layout_height="?android:attr/actionBarSize" <br/>
 *     app:tools_toolbar_title="@string/viewer_toolbar_title" <br/>
 *     app:tools_toolbar_bota_icon="@drawable/xxx" <br/>
 *     app:tools_toolbar_more_icon="@drawable/xxx" <br/>
 *     app:tools_toolbar_search_icon="@drawable/xxx"/> <br/>
 * <p/>
 * custom attrs: <br/>
 * app:tools_toolbar_title="@string/xxx" <br/>
 * app:tools_toolbar_search_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_bota_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_more_icon="@drawable/xxx" <br/>
 * <p/>
 * btn click listener
 * searchBtnClick:
 * @see CPDFToolBar#setSearchBtnClickListener(OnClickListener)
 *
 * botaBtnClick:
 * @see CPDFToolBar#setBoTaBtnClickListener(OnClickListener)
 *
 * moreBtnClick:
 * @see CPDFToolBar#setMoreBtnClickListener(OnClickListener)
 *
 *
 */
public class CPDFToolBar extends FrameLayout {

    private AppCompatTextView tvToolBarTitle;
    private AppCompatImageView ivToolBarSearchBtn;
    private AppCompatImageView ivToolBarBoTaBtn;
    private AppCompatImageView ivToolBarMoreBtn;

    public CPDFToolBar(@NonNull Context context) {
        this(context, null);
    }

    public CPDFToolBar(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPDFToolBar(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initToolBar(context, attrs);

    }

    private void initToolBar(Context context, @Nullable AttributeSet attrs) {
        LayoutInflater.from(getContext()).inflate(R.layout.tools_cpdf_tool_bar, this);
        tvToolBarTitle = findViewById(R.id.tv_tool_bar_title);
        ivToolBarMoreBtn = findViewById(R.id.iv_tool_bar_more);
        ivToolBarSearchBtn = findViewById(R.id.iv_tool_bar_search);
        ivToolBarBoTaBtn = findViewById(R.id.iv_tool_bar_bota);
        initAttributes(context, attrs);
    }

    private void initAttributes(Context context, @Nullable AttributeSet attrs) {
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.CPDFToolBar);
        if (typedArray != null) {
            String toolBarTitle = typedArray.getString(R.styleable.CPDFToolBar_tools_toolbar_title);
            typedArray.getResourceId(R.styleable.CPDFToolBar_tools_toolbar_search_icon, -1);
            if (!TextUtils.isEmpty(toolBarTitle)) {
                tvToolBarTitle.setText(toolBarTitle);
            }
            Drawable searchIconDrawable = ViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_search_icon, R.drawable.tools_ic_search);
            if (searchIconDrawable != null) {
                ivToolBarSearchBtn.setImageDrawable(searchIconDrawable);
            }
            Drawable boTaIconDrawable = ViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_bota_icon, R.drawable.tools_ic_outline);
            if (boTaIconDrawable != null) {
                ivToolBarBoTaBtn.setImageDrawable(boTaIconDrawable);
            }
            Drawable moreIconDrawable = ViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_more_icon, R.drawable.tools_ic_more);
            if (moreIconDrawable != null) {
                ivToolBarMoreBtn.setImageDrawable(moreIconDrawable);
            }

            ViewUtils.applyViewBackground(this);

            typedArray.recycle();
        }
    }



    public void setSearchBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarSearchBtn.setOnClickListener(clickListener);
    }

    public void setBoTaBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarBoTaBtn.setOnClickListener(clickListener);
    }

    public void setMoreBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarMoreBtn.setOnClickListener(clickListener);
    }


}
